"""test_crawler_static_methods.py - Test the babyspider.crawler.Crawler static methods."""

import pytest

from babyspider.crawler import Crawler


def test_crawler_check_parser() -> None:
    """Test the Crawler.check_parser() method."""
    Crawler.check_parser("html.parser")
    with pytest.raises(ValueError):
        Crawler.check_parser("No such parser exists")


def test_crawler_strip_url() -> None:
    """Test the Crawler.strip_url() method."""
    given_and_expected = (
        ("", ""),
        ("http://www.example.com/", "http://www.example.com/"),
        ("http://www.example.com/#chapter-1", "http://www.example.com/"),
        ("http://www.example.com/;one=2&three=4", "http://www.example.com/"),
        ("http://www.example.com/?one=2&three=4", "http://www.example.com/"),
        ("http://www.example.com/;one=2&three=4#hashtag", "http://www.example.com/"),
        ("http://www.example.com/?one=2&three=4#mid-page", "http://www.example.com/"),
        (
            "http://www.example.com/;ay=bee&sea=dee?one=2&three=4#endofline",
            "http://www.example.com/",
        ),
    )
    for given, expected in given_and_expected:
        result = Crawler.strip_url(given)
        assert (
            result == expected
        ), f"Calling Crawler.strip_url({given!r}) should return {expected!r} not {result!r}!"

    # Check handling of unparsable URLs.
    with pytest.raises(ValueError):
        Crawler.strip_url(123)  # type: ignore


def test_crawler_domain_from_url() -> None:
    """Test the Crawler.domain_from_url() method."""
    given_and_expected = (
        ("http://user@www.example.com/", "www.example.com"),
        ("http://www.example.com:443/", "www.example.com"),
        ("http://user:password@www.example.com:443/", "www.example.com"),
        # Without a schema, it's not a URL, so you get None!
        ("", None),
        ("www.example.com/", None),
        ("user:password@www.example.com:443/", None),
    )
    for given, expected in given_and_expected:
        result = Crawler.domain_from_url(given)
        assert (
            result == expected
        ), f"Calling Crawler.domain_from_url({given!r}) should return {expected!r} not {result!r}!"

    # Check handling of unparsable URLs.
    with pytest.raises(ValueError):
        Crawler.domain_from_url(123)  # type: ignore
